#!/usr/local/bin/perl5 -w

# Released under the Msys Public License.  See license.txt for details,
# or http://www-unix.mcs.anl.gov/systems/software/msys/copyright.txt .

use IO::Socket;
use IO::Handle;

$basedir="/sandbox/netflowdb/";    # Where to put the files we make
$fileage=300;                      # Seonds between switching outfiles

# Make a list of routers we will receive flows from

$routers{192005170002}="k";    # kiwi
$routers{130202128001}="b";    # bluto

# Make a list of nexthops that we don't want to record flows from.
#      Why?  Argonne National Laboratory has two border routers,
#            that we call Kiwi and Bluto.  If a flow is going from
#            one to the other, it would be counted twice in our
#            statistics--one for each of the border routers.  So
#            we throw away the one that's going to the other.

$nexthopignore{130202128201}=TRUE;
$nexthopignore{130202128202}=TRUE;

# Convert a UNIX time value into something that we can shove into Oracle
#
#sub timestr {
#	my $t=shift(@_);
#	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($t);
#	$mon++; # localtime() returns 0=Jan, 1=Feb, etc
#	"$mon/$mday/$year-$hour:$min:$sec";
#}

# Convert a UNIX time value into something that we can shove into MySQL

sub timestr {
    my $t=shift(@_);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($t);
    $mon++; # localtime() returns 0=Jan, 1=Feb, etc
    $year+=1900; # $year is the number of years since 1900, 
                 # that is, $year is 123 in year 2023.
    sprintf("%4.4d%2.2d%2.2d%2.2d%2.2d%2.2d",$year,$mon,$mday,$hour,$min,$sec);
}

# Convert an IP address in integer format to a decimal number.  That is,
# IP address 140.221.9.6 is 0x8cdd0906, and we want this function to
# return 140221009006.

sub num2ipnum {
	my $a=shift(@_);
	my $a1=($a&0xff000000)>>24;
    my $a2=($a&0x00ff0000)>>16;
    my $a3=($a&0x0000ff00)>>8;
    my $a4=($a&0x000000ff);
    sprintf("%3.3d%3.3d%3.3d%3.3d",$a1,$a2,$a3,$a4);
}

select((select(STDOUT),$|=1)[0]);

# Open a socket to receive Cisco netflow output packets

my $flows = IO::Socket::INET->new(
				Proto		=> "udp",
				LocalAddr	=> "0.0.0.0",
				LocalPort	=> "(2155)",
			)
			or die "cannot open socket to listen on 2155";


# Open an output file.
#
# Strategy: create $basedir/tmp.pid.unixtime to write.  When $fileage 
#           seconds (or more)
#           have elapsed, close the file and rename it to be
#           netflows.unixtime.

$outfilename=$basedir."/tmp.".$$.".".time();
open(OUTFILE,">".$outfilename) || die "open(OUTFILE): $!";
$fileopentime=time();

# Loop and handle them

FOREVER: while(1) {
	my ($buf,$rcvval);
	$rcvval=$flows->recv($buf,2000,0) || die "recv: $!";
    my $rtrkey=$routers{num2ipnum(unpack('x4 N x8',$rcvval))};
#	printf "Received %d byte packet\n",length($buf);
#	printf "  Router address       : %s\n",
#             num2ipnum(unpack('x4 N x8',$rcvval));
#	printf "  Router Name          : %s\n",$routername{$rtrkey};

	if (!($rtrkey)) {
		printf "UNKNOWN ROUTER ";
        next FOREVER;
    }
	
	my @FLOWS=unpack("a24 a48 a48 a48 a48 a48 a48 a48 a48 a48 a48 a48".
					" a48 a48 a48 a48 a48 a48 a48 a48 a48 a48 a48".
					" a48 a48 a48 a48 a48 a48 a48 a48",$buf); 

	my ($hver,$hcount,$hsysuptime,$hunix_secs,$hunix_nsecs,$hflow_sequence)=
		unpack("n n N N N N x4",shift(@FLOWS));

	next FOREVER if ($hver != 5);
	next FOREVER if ($hcount != ((length($buf)-24)/48));

	# Figure out the time that the router was rebooted, since that is what
    # the sysUptime value in each flow will be based on
	my $basetime=$hunix_secs+($hunix_nsecs/1000000000)-($hsysuptime/1000);

#	printf "   OK packet, flows    : %d\n",$hcount;

    HANDLE_A_PACKET: 
    while(1) {
		my $thisflow=shift(@FLOWS);
		last HANDLE_A_PACKET 
            if ((!defined($thisflow))||length($thisflow)<48);
		my ($fsrc,$fdst,$fnext,$fin,$fout,$fpkts,$focts,$fstrt,$fend,$fsrcp,
			$fdstp,$ftcp,$fprot,$ftos,$fsas,$fdas,$fsmsk,$fdmsk)=
			unpack("N3 n2 N4 n2 x C3 n2 C2 x2",$thisflow);
#		printf "    %s/%2.2d (AS %5.5d) -> %s/%2.2d (AS %5.5d)",
#			num2ipnum($fsrc),$fsmsk,$fsas,num2ipnum($fdst),$fdmsk,$fdas;
#		printf "    %s %s\n",
#			timestr($basetime+($fstrt/1000)),
#			timestr($basetime+($fend/1000));

        next HANDLE_A_PACKET
            if ($nexthopignore{num2ipnum($fnext)});

        printf(OUTFILE "%s\t",$rtrkey);
        printf(OUTFILE "%s\t",num2ipnum($fsrc));
        printf(OUTFILE "%s\t",num2ipnum($fdst));
        printf(OUTFILE "%s\t",num2ipnum($fnext));
        printf(OUTFILE "%d\t",$fin);
        printf(OUTFILE "%d\t",$fout);
        printf(OUTFILE "%d\t",$fpkts);
        printf(OUTFILE "%d\t",$focts);
        printf(OUTFILE "%s\t",timestr($basetime+($fstrt/1000)));
        printf(OUTFILE "%s\t",timestr($basetime+($fend/1000)));
        printf(OUTFILE "%d\t",$fsrcp);
        printf(OUTFILE "%d\t",$fdstp);
        printf(OUTFILE "%d\t",$ftcp);
        printf(OUTFILE "%d\t",$fprot);
        printf(OUTFILE "%d\t",$ftos);
        printf(OUTFILE "%d\t",$fsas);
        printf(OUTFILE "%d\t",$fdas);
        printf(OUTFILE "%d\t",$fsmsk);
        printf(OUTFILE "%d\t",$fdmsk);
        printf(OUTFILE "\n");

    }

	printf ("%2.2d%s ",$hcount,$rtrkey);

    if (time() > $fileopentime + $fileage) {
        close OUTFILE;
        $finalname=$basedir."/out.".$fileopentime;
        rename($outfilename,$finalname);
        printf("\n%s -> %s\n",$outfilename,$finalname);
        $outfilename=$basedir."/tmp.".$$.".".time();
        open(OUTFILE,">".$outfilename) || die "open(OUTFILE): $!";
        $fileopentime=time();
    }
}
